@@ -6,5 +6,13 @@ from autots.datasets._base import load_monthly
 from autots.datasets._base import load_yearly
 from autots.datasets._base import load_hourly
 from autots.datasets._base import load_weekly
+from autots.datasets._base import load_weekdays
 
-__all__ = ['load_daily', 'load_monthly', 'load_yearly', 'load_hourly', 'load_weekly']
+__all__ = [
+    'load_daily',
+    'load_monthly',
+    'load_yearly',
+    'load_hourly',
+    'load_weekly',
+    'load_weekdays',
+]
