@@ -6,6 +6,7 @@ import pandas as pd
 def df_cleanup(
     df_wide,
     frequency: str = "infer",
+    prefill_na: str = None,
     na_tolerance: float = 0.999,
     drop_data_older_than_periods: int = 100000,
     drop_most_recent: int = 0,
@@ -56,10 +57,22 @@ def df_cleanup(
     # drop older data, because too much of a good thing...
     if str(drop_data_older_than_periods).isdigit():
         if int(drop_data_older_than_periods) < df_wide.shape[0]:
-            if verbose >= 0:
+            if verbose >= 1:
                 print("Old data dropped by `drop_data_older_than_periods`.")
             df_wide = df_wide.tail(int(drop_data_older_than_periods))
 
+    # fill NaN now if asked:
+    if prefill_na is not None:
+        if str(prefill_na).isdigit():
+            df_wide = df_wide.fillna(float(prefill_na))
+        elif prefill_na == "mean":
+            df_wide = df_wide.fillna(df_wide.mean(axis=0))
+        elif prefill_na == "median":
+            df_wide = df_wide.fillna(df_wide.median(axis=0))
+        else:
+            if verbose >= 0:
+                print("WARNING: prefill_na method {prefill_na} not recognized.")
+
     # remove series with way too many NaNs
     na_tolerance = abs(float(na_tolerance))
     na_tolerance = 1 if na_tolerance > 1 else na_tolerance
@@ -141,6 +154,7 @@ class NumericTransformer(object):
         categorical_fillna (str): how to fill NaN for categorical variables (numeric NaN are unaltered)
             "ffill" - uses forward and backward filling to supply na values
             "indicator" or anything else currently results in all missing replaced with str "missing_value"
+        handle_unknown (str): passed through to scikit-learn OrdinalEncoder
         verbose (int): greater than 0 to print some messages
     """
 
@@ -148,11 +162,13 @@ class NumericTransformer(object):
         self,
         na_strings: list = ['', ' '],  # 'NULL', 'NA', 'NaN', 'na', 'nan'
         categorical_fillna: str = "ffill",
+        handle_unknown: str = 'use_encoded_value',
         verbose: int = 0,
     ):
         self.na_strings = na_strings
         self.verbose = verbose
         self.categorical_fillna = categorical_fillna
+        self.handle_unknown = handle_unknown
         self.categorical_flag = False
         self.needs_transformation = True
 
@@ -192,7 +208,9 @@ class NumericTransformer(object):
                 if self.categorical_fillna == "ffill":
                     df_enc = df_enc.fillna(method='ffill').fillna(method='bfill')
                 df_enc = df_enc.fillna('missing_value')
-                self.cat_transformer = OrdinalEncoder()
+                self.cat_transformer = OrdinalEncoder(
+                    handle_unknown=self.handle_unknown, unknown_value=np.nan
+                )
                 # the + 1 makes it compatible with remove_leading_zeroes
                 df_enc = self.cat_transformer.fit_transform(df_enc) + 1
                 # df_enc = self.cat_transformer.transform(df_enc) + 1
